%   Multi-scale HDR tonemapping using the WLS edge-preserving filter.
%
%   This script computes a multi-scale edge-preserving decomposition
%   of an HDR image, using the weighted least squares (WLS) optimization
%   framework, as described in Farbman, Fattal, Lischinski, and Szeliski,
%   "Edge-Preserving Decompositions for Multi-Scale Tone and Detail
%   Manipulation", ACM Transactions on Graphics, 27(3), August 2008.
%
%   The decomposition is then used to produce two different tonemapped
%   looks by recombining the detail layers in different manners.

%% Load
filename = 'desk.hdr';

hdr = double(hdrread(filename));
I = 0.299*hdr(:,:,1) + 0.587*hdr(:,:,2) + 0.114*hdr(:,:,3) + 1e-6;
R = hdr(:,:,1) ./ I;
G = hdr(:,:,2) ./ I;
B = hdr(:,:,3) ./ I;

LL = log(I);

%% Filter
u0 = LL;
u1 = wlsFilter(LL, 0.125, 1.2, LL );
u2 = wlsFilter(LL, 1.0,   1.2, LL );
u3 = wlsFilter(LL, 8.0,   1.2, LL );

%% Compute detail layers
d0 = u0-u1;
d1 = u1-u2;
d2 = u2-u3;

%% Recombine the layers together while moderately boosting multiple scale detail
look = 'Balanced'; 
w0 = 1.0;
w1 = 1.0;
w2 = 1.6;
w3 = 0.4;
sat = 0.9;
exposure = 7.0;
gamma = 1.0/1.6;

cLL = w0*d0 + w1*d1 + w2*d2 + w3*u3;

%% Convert back to RGB
Inew = exp(cLL);
sI = sort(Inew(:));
mx = sI(round(length(sI) * (99.9/100)));
Inew = Inew/mx;

rgb = composeIRGB(exposure*Inew, R, G, B, sat, gamma);
imtool(rgb);
imwrite(rgb, ['desk_' look '.png']);

%% Recombine the layers together with stronger emphasis on fine detail
look = 'StrongFine';
w0 = 2.0;
w1 = 0.8;
w2 = 0.7;
w3 = 0.2;
sat = 0.6;
exposure = 2.5;
gamma = 1.0;
%w0 = 3.0; w1 = 1.0; w2 = 1.0; w3 = 0.45;

cLL = w0*d0 + w1*d1 + w2*d2 + w3*u3;

%% Convert back to RGB
Inew = exp(cLL);
sI = sort(Inew(:));
mx = sI(round(length(sI) * (99.9/100)));
Inew = Inew/mx;

rgb = composeIRGB(exposure*Inew, R, G, B, sat, gamma);
imtool(rgb);
imwrite(rgb, ['desk_' look '.png']);
